/* The MIT License (MIT)

Copyright (c) 2016, Markus Hohloch

Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of 
the Software, and to permit persons to whom the Software is furnished to do so, 
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS 
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR 
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

--------------------------------------------------------
--  DDL for Sequence JR_LOG_SEQ
--------------------------------------------------------
create sequence jasper.jr_log_seq  minvalue 1 maxvalue 999999999999999999999999999 increment by 1 start with 1 cache 20 noorder  nocycle;

--------------------------------------------------------
--  DDL for Table JR_LOG
--------------------------------------------------------
create table jasper.jr_log (
  id number(10,0), 
	log_level varchar2(10 char), 
	code varchar2(20 char), 
	message varchar2(4000 char), 
	time timestamp (6), 
	category varchar2(50 char)
);

--------------------------------------------------------
--  DDL for Index JR_LOG_IX1
--------------------------------------------------------
create index jasper.jr_log_ix1 on jasper.jr_log (category);

--------------------------------------------------------
--  DDL for Index JR_LOG_PK
--------------------------------------------------------
create unique index jasper.jr_log_pk on jasper.jr_log (id);
--------------------------------------------------------
--  Constraints for Table JR_LOG
--------------------------------------------------------
alter table jasper.jr_log modify (id not null enable);
alter table jasper.jr_log modify (log_level not null enable);
alter table jasper.jr_log add constraint dbs_log_pk primary key (id) enable;

--------------------------------------------------------
--  DDL for Trigger JR_LOG_BI_TR
--------------------------------------------------------
create or replace trigger jasper.jr_log_bi_tr before insert on jr_log               
  for each row  
begin   
  if :new.id is null then 
    select jr_log_seq.nextval into :new.id from dual; 
  end if; 
end;
/
alter trigger jasper.jr_log_bi_tr enable;

--------------------------------------------------------
--  DDL for Package JR_SERVER
--------------------------------------------------------
create or replace package jasper.jr_server is

  -- Type definitions
  type type_parameter is table of varchar2(1000) index by varchar2(50);
  
  -- Configuration variables
  host             varchar2(255);
  port             number(5,0);
  path             varchar2(255);
  username         varchar2(255);
  password         varchar2(255);
  https            boolean;
  reports_folder   varchar2(255);
  output_format    varchar2(50);
  -- get_report() options
  report_file_name varchar2(255);
  show_inline      boolean;
  
  -- Download report
  procedure call_report(p_report_name varchar2);
  
  -- Download report with parameters
  procedure call_report(p_report_name varchar2, p_parameter type_parameter);
  
  -- Get report as binary file 
  function get_report(p_report_name varchar2) return blob;
  
  -- Get report as binary file with parameters
  function get_report(p_report_name varchar2, p_parameter type_parameter) return blob;
	
end jr_server;

/* 
-- setup grants and acl's for this package

grant execute on utl_inaddr to jasper;
grant execute on utl_http to jasper;

begin
  if dbms_db_version.ver_le_10_2 then
    null;
  else 
    begin
      dbms_network_acl_admin.drop_acl(
        acl =>         'jasperreports-server.xml'
      );
    exception 
      when others then null; -- ACL existiert noch nicht
    end;
    dbms_network_acl_admin.create_acl(
      acl =>         'jasperreports-server.xml',
      description => 'Netzwerk-Connects von Schema jasper aus JasperReports Server',
      principal =>   'JASPER', -- PARSING SCHEMA der Anwendung
      is_grant =>    true,
      privilege =>   'connect'
    );
    DBMS_NETWORK_ACL_ADMIN.ADD_PRIVILEGE(
      acl =>         'jasperreports-server.xml',
      principal =>   'JASPER', -- PARSING SCHEMA der Anwendung
      is_grant  =>   true,
      privilege =>   'resolve'
    );
    dbms_network_acl_admin.assign_acl(
      acl =>         'jasperreports-server.xml',
      host =>        'localhost',
      lower_port =>  8080,
      upper_port =>  8080
    );
  end if;
end;
/
show error
    
commit
/

*/

/
--------------------------------------------------------
--  DDL for Package Body JR_SERVER
--------------------------------------------------------
create or replace package body jasper.jr_server is

	-- private variables
	C_USER_AGENT constant varchar2(255) := 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/A.B (KHTML, like Gecko) Chrome/X.Y.Z.W Safari/A.B.';

	-- private functions
	function get_url(p_report_name varchar2, p_parameter type_parameter) return varchar2
	is
		l_url varchar2(4000);
		l_key varchar2(1000);
	begin
		if https then
			l_url := 'https://';
		else
			l_url := 'http://';
		end if;
		
    --don't forget: grant execute on utl_http to jasper
		sys.utl_http.set_wallet('file:/var/opt/oracle/root_certs');

		l_url := l_url || host;
		l_url := l_url || ':' || port;
		l_url := l_url || path;
		l_url := l_url || '&' || 'j_username=' || username;
		l_url := l_url || '&' || 'j_password=' || password;
		l_url := l_url || '&' || 'parentFolderUri=' || reports_folder;
		l_url := l_url || '&' || 'reportUnit=' || reports_folder || p_report_name;
		l_url := l_url || '&' || 'output=' || output_format;

    l_key := p_parameter.first;    
    while l_key is not null loop    
      l_url := l_url || '&' || l_key || '=' || p_parameter(l_key);
      l_key := p_parameter.next(l_key);  
    end loop;
    
    insert into jr_log (log_level, message) values ('1',l_url);
    commit;

		return l_url;
	end get_url;

	
	-- report download
	--------------------------------------------------------------------------------  
	procedure call_report(p_report_name varchar2, p_parameter type_parameter)
		is
		l_url       varchar2(4000);
		l_request   sys.utl_http.req;
		l_response  sys.utl_http.resp;
		l_file      blob;
		l_download  raw(32767);
	begin
		l_url := get_url(p_report_name, p_parameter);

		l_request := sys.utl_http.begin_request(l_url);
		sys.utl_http.set_header(l_request, 'User-Agent', C_USER_AGENT);
		l_response := sys.utl_http.get_response(l_request);

		dbms_lob.createtemporary(l_file, true, dbms_lob.session);

		loop
			begin
				sys.utl_http.read_raw(l_response, l_download);
				dbms_lob.writeappend(l_file, utl_raw.length(l_download), l_download);
			exception when sys.utl_http.end_of_body then
				exit;
			end;
		end loop;

		sys.utl_http.end_response(l_response);

		owa_util.mime_header('application/' || output_format, false);
		htp.p('Content-length: ' || dbms_lob.getlength(l_file));
    if show_inline then
      htp.p('Content-Disposition: inline;     filename=' || report_file_name || '.'|| output_format ||'');
    else
      htp.p('Content-Disposition: attachment; filename=' || report_file_name || '.'|| output_format ||'');
    end if;
		owa_util.http_header_close;

		wpg_docload.download_file(l_file);

		dbms_lob.freetemporary(l_file);
    
    -- futher rendering apex page not necessary
    apex_application.stop_apex_engine;
    
	end call_report;
  
  -- same procedure, but without parameters
  procedure call_report(p_report_name varchar2)
  as
    l_parameter type_parameter;
  begin
    call_report(p_report_name, l_parameter);
  end call_report;

  -- see spec
	function get_report(p_report_name varchar2, p_parameter type_parameter) return blob
		is
		l_url       varchar2(4000);
		l_request   sys.utl_http.req;
		l_response  sys.utl_http.resp;
		l_file      blob;
		l_download  raw(32767);
	begin
    insert into jr_log (log_level, message) values ('1', p_report_name);
    
		l_url := get_url(p_report_name, p_parameter);
    
    insert into jr_log (log_level, message) values ('1', l_url);

		l_request := sys.utl_http.begin_request(l_url);
		sys.utl_http.set_header(l_request, 'user-agent', C_USER_AGENT);
		l_response := sys.utl_http.get_response(l_request);

		dbms_lob.createtemporary(l_file, true, dbms_lob.session);

		loop
			begin
				sys.utl_http.read_raw(l_response, l_download);
				dbms_lob.writeappend(l_file, utl_raw.length(l_download), l_download);
			exception when sys.utl_http.end_of_body then
				exit;
			end;
		end loop;

		sys.utl_http.end_response(l_response);

		return l_file;
	end get_report;

  -- see spec
  function get_report(p_report_name varchar2) return blob
  as
    l_parameter type_parameter;
  begin
    return get_report(p_report_name, l_parameter);
  end get_report;

	-- default configuration
	begin
		host             := 'localhost';
		port             := 8080;
		path             := '/jasperserver/flow.html?_flowId=viewReportFlow';
		username         := 'johndoe';
		password         := 'changeme';
		https            := false;
		reports_folder   := '/apex/';
		output_format    := 'pdf';
    
    
    -- configuration options for get_report()
		-- default download file name
    report_file_name := 'report';
    -- Force browser to show report file inline (true) or as download (false)
    show_inline      := false; 

end jr_server;

/
